#include "canvas/BackGround.hpp"
#include "canvas/BackGroundColors.hpp"
#include "Math.hpp"

#include <raylib.h>
#include <rlgl.h>
#include <raymath.h>

// Public
void BackGround::init(int size)
{
  this->size = size;
}

void BackGround::newGen()
{
  starBuff.reset();
  mountenBuff.reset();
  m_moon = {GetRandomFloat(), GetRandomFloat(), GetRandomFloat()};

  BackGroundColors::setColor();

  draw();
}

void BackGround::draw()
{
  ClearBackground(BackGroundColors::backGroundColor);
  drawStarts();
  drawMoon();
  drawMounten(20, (int)(mounten1min * size), (int)(mounten1max * size), BackGroundColors::backMountenColor);
  drawMounten(21, (int)(mounten2min * size), (int)(mounten2max * size), ColorLerp(BackGroundColors::backMountenColor, BackGroundColors::frontMountenColor, colorRatio1));
  drawMounten(23, (int)(mounten3min * size), (int)(mounten3max * size), ColorLerp(BackGroundColors::backMountenColor, BackGroundColors::frontMountenColor, colorRatio2));
}

void BackGround::drawStarts()
{
  rlSetTexture(1);
  rlBegin(RL_TRIANGLES);
  rlNormal3f(0.0f, 0.0f, 1.0f);
  bool star = true;

  for (size_t i = 0; i < numOfStarts; i++)
  {
    int x = starBuff.next() * size;
    int y = starBuff.next() * size;
    int weight = starBuff.next() * 3 + 1;
    float alph = Normalize(starBuff.next(), 0.04f, 0.8f);
    Color color = ColorLerp(BackGroundColors::backGroundColor, BackGroundColors::starColor, alph);

    rlColor4ub(color.r, color.g, color.b, color.a);
    if (star)
    {
      // topLeft
      rlVertex2f(x, y);
      // bottomLeft
      rlVertex2f(x, y + weight);
      // topRight
      rlVertex2f(x + weight, y);
    }
    else
    {
      // bottomRight
      rlVertex2f(x + weight, y + weight);
      // topRight
      rlVertex2f(x + weight, y);
      // bottomLeft
      rlVertex2f(x, y + weight);
    }
    star = !star;
  }
  rlEnd();
  rlSetTexture(0);
}

void BackGround::drawMoon()
{
  int r = ((m_moon.size * (maxSizeOfMoon - minSizeOfMoon)) + minSizeOfMoon) * size;
  int xpos = Lerp(size * moonXOffset, size - size * moonXOffset, m_moon.x);
  int ypos = Lerp(size * moonXOffset, maxYPosOfMoon * size, m_moon.y);

  int ring1 = (float)r * bigRingRatio;
  int ring2 = (float)r * smallRingRatio;

  DrawCircle(xpos, ypos, r + ring1, ColorLerp(BackGroundColors::backGroundColor, BackGroundColors::moonColor, bigRingBlend));
  DrawCircle(xpos, ypos, r + ring2, ColorLerp(BackGroundColors::backGroundColor, BackGroundColors::moonColor, smallRingBlend));
  DrawCircle(xpos, ypos, r, BackGroundColors::moonColor);
}

void BackGround::drawMounten(size_t mountenSegments, int min, int max, Color color)
{
  int x = 0;
  int diff = size / (mountenSegments - 1);

  float p = mountenBuff.next();

  int y = Lerp(min, max, p);
  rlSetTexture(1);
  rlBegin(RL_TRIANGLES);
  rlNormal3f(0.0f, 0.0f, 1.0f);
  rlColor4ub(color.r, color.g, color.b, color.a);

  for (size_t i = 1; i <= mountenSegments; i++)
  {
    // topLeft
    rlVertex2f(x, y);
    // bottomLeft
    rlVertex2f(x, size);

    p = mountenBuff.next();
    y = Lerp(min, max, p);
    x += diff;
    // topRight
    rlVertex2f(x, y);

    // bottomRight
    rlVertex2f(x, size);
    // topRight
    rlVertex2f(x, y);

    rlVertex2f(x - diff, size);
  }
  rlEnd();
  rlSetTexture(0);
}